package org.opendolphin.core.client.comm

import org.opendolphin.core.Attribute
import org.opendolphin.core.client.ClientModelStore
import org.opendolphin.core.comm.BaseValueChangedCommand
import org.opendolphin.core.comm.ChangeAttributeMetadataCommand
import org.opendolphin.core.comm.ValueChangedCommand

import java.beans.PropertyChangeEvent
import java.beans.PropertyChangeListener

class AttributeChangeListener implements PropertyChangeListener {

    ClientModelStore clientModelStore
    ClientConnector clientConnector

    void propertyChange(PropertyChangeEvent evt) {
        if (evt.propertyName == Attribute.DIRTY_PROPERTY) {
            // ignore
        } else if (evt.propertyName == Attribute.VALUE) {
            if (evt.oldValue == evt.newValue) return
            clientConnector.send constructValueChangedCommand(evt)
            List<Attribute> attributes = clientModelStore.findAllAttributesByQualifier(evt.source.qualifier)
            attributes.each { it.value = evt.newValue }
        } else if (evt.propertyName == Attribute.BASE_VALUE) {
            if (evt.oldValue == evt.newValue) return
            clientConnector.send constructBaseValueChangedCommand(evt)
            List<Attribute> attributes = clientModelStore.findAllAttributesByQualifier(evt.source.qualifier)
            attributes.each { it.rebase() }
        } else {
            // we assume the change is on a metadata property such as qualifier
            clientConnector.send constructChangeAttributeMetadataCommand(evt)
        }
    }

    private ValueChangedCommand constructValueChangedCommand(PropertyChangeEvent evt) {
        new ValueChangedCommand(
                attributeId: evt.source.id,
                oldValue: evt.oldValue,
                newValue: evt.newValue
        )
    }

    private BaseValueChangedCommand constructBaseValueChangedCommand(PropertyChangeEvent evt) {
        new BaseValueChangedCommand(
                attributeId: evt.source.id
        )
    }

    private ChangeAttributeMetadataCommand constructChangeAttributeMetadataCommand(PropertyChangeEvent evt) {
        new ChangeAttributeMetadataCommand(
                attributeId: evt.source.id,
                metadataName: evt.propertyName,
                value: evt.newValue
        )
    }

}
